@@ -2,12 +2,16 @@ import base64
 import sys
 import os
 import fnmatch
-import cPickle
 import gzip
 import logging
 from random import randint
-import compressor
-import ppd
+import pyppd.compressor
+import pyppd.ppd
+
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
 def archive(ppds_directory):
     """Returns a string with the decompressor, its dependencies and the archive.
@@ -27,8 +31,8 @@ def archive(ppds_directory):
     template = read_file_in_syspath("pyppd/pyppd-ppdfile.in")
     compressor_py = read_file_in_syspath("pyppd/compressor.py")
 
-    template = template.replace("@compressor@", compressor_py)
-    template = template.replace("@ppds_compressed_b64@", ppds_compressed_b64)
+    template = template.replace(b"@compressor@", compressor_py)
+    template = template.replace(b"@ppds_compressed_b64@", ppds_compressed_b64)
 
     return template
 
@@ -44,7 +48,7 @@ def compress(directory):
     returns a compressed pickle dump of it.
 
     """
-    ppds = ""
+    ppds = b""
     ppds_index = {}
     abs_directory = os.path.abspath(directory)
 
@@ -57,16 +61,16 @@ def compress(directory):
             # We don't want the .gz extension in our filename
             ppd_filename = ppd_filename[:-3]
         else:
-            ppd_file = open(ppd_path).read()
+            ppd_file = open(ppd_path, 'rb').read()
 
         start = len(ppds)
         length = len(ppd_file)
         logging.debug('Found %s (%d bytes).' % (ppd_path, length))
 
-        ppd_parsed = ppd.parse(ppd_file, ppd_filename)
-        ppd_descriptions = map(str, ppd_parsed)
+        ppd_parsed = pyppd.ppd.parse(ppd_file, ppd_filename)
+        ppd_descriptions = [p.__str__() for p in ppd_parsed]
         ppds_index[ppd_parsed[0].uri] = (start, length, ppd_descriptions)
-        logging.debug('Adding %d entry(ies): %s.' % (len(ppd_descriptions), map(str, ppd_parsed)))
+        logging.debug('Adding %d entry(ies): %s.' % (len(ppd_descriptions), ppd_descriptions))
         ppds += ppd_file
 
     if not ppds:
@@ -74,9 +78,9 @@ def compress(directory):
         return None
 
     logging.info('Compressing archive.')
-    ppds_index['ARCHIVE'] = compressor.compress(ppds)
+    ppds_index['ARCHIVE'] = pyppd.compressor.compress(ppds)
     logging.info('Generating and compressing pickle dump.')
-    ppds_pickle = compressor.compress(cPickle.dumps(ppds_index))
+    ppds_pickle = pyppd.compressor.compress(pickle.dumps(ppds_index))
 
 
     return ppds_pickle
@@ -91,7 +95,7 @@ def read_file_in_syspath(filename):
     last_exception = None
     for path in sys.path:
         try:
-            return open(path + "/" + filename).read()
+            return open(path + "/" + filename, 'rb').read()
         except IOError as ex:
             last_exception = ex
             continue
