@@ -1,13 +1,13 @@
 from subprocess import Popen, PIPE
 
 def compress(value):
-    """Compresses a string with the xz binary"""
+    """Compresses a byte array with the xz binary"""
 
     process = Popen(["xz", "--compress", "--force"], stdin=PIPE, stdout=PIPE)
     return process.communicate(value)[0]
 
 def decompress(value):
-    """Decompresses a string with the xz binary"""
+    """Decompresses a byte array with the xz binary"""
 
     process = Popen(["xz", "--decompress", "--stdout", "--force"],
                     stdin=PIPE, stdout=PIPE)
